@@ -3,7 +3,7 @@ package POE::Component::Server::TCP;
 use strict;
 
 use vars qw($VERSION);
-$VERSION = '1.354'; # NOTE - Should be #.### (three decimal places)
+$VERSION = '1.358'; # NOTE - Should be #.### (three decimal places)
 
 use Carp qw(carp croak);
 use Socket qw(INADDR_ANY inet_ntoa inet_aton AF_INET AF_UNIX PF_UNIX);
@@ -116,6 +116,7 @@ sub new {
   my $session_type        = delete $param{SessionType};
   my $session_params      = delete $param{SessionParams};
   my $server_started      = delete $param{Started};
+  my $server_stopped      = delete $param{Stopped};
   my $listener_args       = delete $param{ListenerArgs};
 
   $listener_args = [] unless defined $listener_args;
@@ -299,9 +300,11 @@ sub new {
               my $socket = $_[ARG0];
               if ($client_pre_connect) {
                 $socket = $client_pre_connect->(@_);
-                unless (fileno($socket)) {
-                  # TODO - Error callback?  Disconnected callback?
-                  # TODO - Should we do this before starting the child?
+                unless (defined($socket) and ref($socket) and fileno($socket)) {
+                  # TODO - The user ought to know what's going on
+                  # here, since it's triggered by something their
+                  # callback has done.  Should we expose a callback
+                  # anyway to avoid potential confusion?
                   return;
                 }
               }
@@ -480,7 +483,7 @@ sub new {
       # We accepted a connection.  Do something with it.
       tcp_server_got_connection => $accept_callback,
 
-      # conncurrency on close.
+      # concurrency on close.
       disconnected => sub {
         $_[HEAP]->{connections}--;
         DEBUG and warn(
@@ -539,6 +542,7 @@ sub new {
       # Dummy states to prevent warnings.
       _stop   => sub {
         DEBUG and warn "$$: $_[HEAP]->{alias} _stop";
+        $server_stopped and $server_stopped->(@_);
         undef($accept_session_id);
         return 0;
       },
@@ -724,6 +728,10 @@ set to a copy of the values in the server's C<ListenerArgs>
 constructor parameter.  The other parameters are standard for
 POE::Session's _start handlers.
 
+The component's C<Stopped> callback is invoked at the beginning of the
+master session's _stop routine. The parameters are standard for
+POE::Session's _stop handlers.
+
 The component's C<Error> callback is invoked when the server has a
 problem listening for connections.  C<Error> may also be called if the
 component's default acceptor has trouble accepting a connection.
@@ -744,8 +752,9 @@ connection---and not in the master listening session.  This has been a
 major point of confusion.  We welcome suggestions for making this
 clearer.
 
-Z<TODO - Document some of the implications of having each connection
-handled by a separate session.>
+=for comment
+TODO - Document some of the implications of having each connection
+handled by a separate session.
 
 The component's C<ClientInput> callback defines how child sessions
 will handle input from their clients.  Its parameters are that of
@@ -762,15 +771,21 @@ included in C<ClientConnected>'s parameters as @_[ARG0..$#_].
     ...
   }
 
-C<ClientPreConnect> is called before C<ClientConnected>, and it has
-different parameters: $_[ARG0] contains a copy of the client socket
-before it's given to POE::Wheel::ReadWrite for management.  Most HEAP
-members are set, except of course $_[HEAP]{client}, because the
-POE::Wheel::ReadWrite has not yet been created yet.
-C<ClientPreConnect> may enable SSL on the socket, using
-POE::Component::SSLify.  C<ClientPreConnect> must return a valid
-socket to complete the connection; the client will be disconnected if
-anything else is returned.
+C<ClientPreConnect> is called before C<ClientConnected>, and its
+purpose is to allow programs to reject connections or condition
+sockets before they're given to POE::Wheel::ReadWrite for management.
+
+The C<ClientPreConnect> handler is called with the client socket in
+$_[ARG0], and its return value is significant.  It must return a
+valid client socket if the connection is acceptable.  It must return
+undef to reject the connection.
+
+Most $_[HEAP] values are valid in the C<ClientPreConnect> handler.
+Obviously, $_[HEAP]{client} is not because that wheel hasn't been
+created yet.
+
+In the following example, the C<ClientPreConnect> handler returns the
+client socket after it has been upgraded to an SSL connection.
 
   sub handle_client_pre_connect {
 
@@ -848,7 +863,8 @@ They are covered briefly again below.
 These constructor parameters affect POE::Component::Server::TCP's main
 listening session.
 
-Z<TODO - Document the shutdown procedure somewhere.>
+=for comment
+TODO - Document the shutdown procedure somewhere.
 
 =head4 Acceptor
 
@@ -885,7 +901,8 @@ supports.  At the time of this writing, that may be a dotted IPv4
 quad, an IPv6 address, a host name, or a packed Internet address.  See
 also L</Hostname>.
 
-Z<TODO - Example, using the lines below.>
+=for comment
+TODO - Example, using the lines below.
 
   Address => '127.0.0.1'   # Localhost IPv4
   Address => "::1"         # Localhost IPv6
@@ -966,8 +983,7 @@ parameters as discussed there.
 
 A default error handler will be provided if Error is omitted.  The
 default handler will log the error to STDERR and shut down the server.
-Active connections will be permitted to to complete their
-transactions.
+Active connections will be permitted to complete their transactions.
 
   Error => sub {
     my ($syscall_name, $err_num, $err_str) = @_[ARG0..ARG2];
@@ -998,7 +1014,8 @@ will not reach these handlers.
 
 If POE::Kernel::ASSERT_USAGE is enabled, the constructor will croak() if it
 detects a state that it uses internally. For example, please use the "Started"
-callback if you want to specify your own "_start" event.
+and "Stopped" callbacks if you want to specify your own "_start" and "_stop"
+events respectively.
 
 =head4 ObjectStates
 
@@ -1012,7 +1029,8 @@ will not reach these handlers.
 
 If POE::Kernel::ASSERT_USAGE is enabled, the constructor will croak() if it
 detects a state that it uses internally. For example, please use the "Started"
-callback if you want to specify your own "_start" event.
+and "Stopped" callbacks if you want to specify your own "_start" and "_stop"
+events respectively.
 
 =head4 PackageStates
 
@@ -1027,7 +1045,8 @@ will not reach these handlers.
 
 If POE::Kernel::ASSERT_USAGE is enabled, the constructor will croak() if it
 detects a state that it uses internally. For example, please use the "Started"
-callback if you want to specify your own "_start" event.
+and "Stopped" callbacks if you want to specify your own "_start" and "_stop"
+events respectively.
 
 =head4 Port
 
@@ -1046,10 +1065,17 @@ main server session's context.  It notifies the server that it has
 fully started.  The callback's parameters are the usual for a
 session's _start handler.
 
+=head4 Stopped
+
+C<Stopped> sets an optional callback that will be invoked within the
+main server session's context.  It notifies the server that it has
+fully stopped.  The callback's parameters are the usual for a
+session's _stop handler.
+
 =head4 ListenerArgs
 
 C<ListenerArgs> is passed to the listener session as the C<args> parameter.  In
-other words, it must be an arrayref, and the values are are passed into the
+other words, it must be an arrayref, and the values are passed into the
 C<Started> handler as ARG0, ARG1, etc.
 
 =head3 Connection Session Configuration
@@ -1410,11 +1436,12 @@ Some use cases require different session classes for the listener and
 the connection handlers.  This isn't currently supported.  Please send
 patches. :)
 
-Z<TODO - Document that Reuse is set implicitly.>
+=for comment
+TODO - Document that Reuse is set implicitly.
 
 =head1 AUTHORS & COPYRIGHTS
 
-POE::Component::Server::TCP is Copyright 2000-2009 by Rocco Caputo.
+POE::Component::Server::TCP is Copyright 2000-2013 by Rocco Caputo.
 All rights are reserved.  POE::Component::Server::TCP is free
 software, and it may be redistributed and/or modified under the same
 terms as Perl itself.
