@@ -4,7 +4,7 @@
 package POE::Resource::Signals;
 
 use vars qw($VERSION);
-$VERSION = '1.354'; # NOTE - Should be #.### (three decimal places)
+$VERSION = '1.358'; # NOTE - Should be #.### (three decimal places)
 
 # These methods are folded into POE::Kernel;
 package POE::Kernel;
@@ -101,12 +101,14 @@ use vars (
 # already been queued.
 my $polling_for_signals = 0;
 
-# A flag determining whether there are child processes.
-use constant BASE_SIGNAL_COUNT => (
-  exists($INC{'Apache.pm'}) ? 0 : ( USE_SIGCHLD ? 0 : 1 )
-);
+# There may be latent subprocesses in some environments.
+# Or we may need to "always loop once" if we're polling for SIGCHLD.
+# This constant lets us define those exceptional cases.
+# We had some in the past, but as of 2013-10-06 we seem to have
+# eliminated those special cases.
+use constant BASE_SIGCHLD_COUNT => 0;
 
-my $kr_has_child_procs = BASE_SIGNAL_COUNT;
+my $kr_has_child_procs = BASE_SIGCHLD_COUNT;
 
 # A list of special signal types.  Signals that aren't listed here are
 # benign (they do not kill sessions at all).  "Terminal" signals are
@@ -199,7 +201,7 @@ sub _data_sig_reset_procs {
   # least once.  Starts false when running in an Apache handler so our
   # SIGCHLD hijinks don't interfere with the web server.
   $self->_data_sig_cease_polling();
-  $kr_has_child_procs = BASE_SIGNAL_COUNT;
+  $kr_has_child_procs = BASE_SIGCHLD_COUNT;
 }
 
 
@@ -252,6 +254,22 @@ sub _data_sig_finalize {
     }
   }
 
+  if ($kr_has_child_procs) {
+    _warn "!!! Kernel has $kr_has_child_procs child process(es).\n";
+  }
+
+  if ($polling_for_signals) {
+    _warn "!!! Finalizing signals while polling is active.\n";
+  }
+
+  if (USE_SIGNAL_PIPE and $self->_data_sig_pipe_has_signals()) {
+    _warn "!!! Finalizing signals while signal pipe contains messages.\n";
+  }
+
+  if (exists $kr_signals{CHLD}) {
+    _warn "!!! Finalizing signals while a blanket _child signal is watched.\n";
+  }
+
   %_safe_signals = ();
 
   unless (RUNNING_IN_HELL) {
@@ -572,7 +590,7 @@ sub _data_sig_enqueue_poll_event {
 
     $self->_data_ev_enqueue(
       $self, $self, EN_SCPOLL, ET_SCPOLL, [ $signal ],
-      __FILE__, __LINE__, undef, time(),
+      __FILE__, __LINE__, undef
     );
   } else {
     return if $self->_data_ses_count() < 1;
@@ -580,7 +598,7 @@ sub _data_sig_enqueue_poll_event {
 
     $self->_data_ev_enqueue(
       $self, $self, EN_SCPOLL, ET_SCPOLL, [ $signal ],
-      __FILE__, __LINE__, undef, time() + POE::Kernel::CHILD_POLLING_INTERVAL(),
+      __FILE__, __LINE__, undef, walltime(), POE::Kernel::CHILD_POLLING_INTERVAL(),
     );
   }
 }
@@ -594,7 +612,7 @@ sub _data_sig_handle_poll_event {
 
   if (TRACE_SIGNALS) {
     _warn(
-      "<sg> POE::Kernel is polling for signals at " . time() .
+      "<sg> POE::Kernel is polling for signals at " . monotime() .
       (USE_SIGCHLD ? " due to SIGCHLD" : "")
     );
   }
@@ -659,7 +677,7 @@ sub _data_sig_reap_pids {
             $self->_data_ev_enqueue(
               $ses_rec->[PID_SESSION], $self, $ses_rec->[PID_EVENT], ET_SIGCLD,
               [ 'CHLD', $pid, $?, @{$ses_rec->[PID_ARGS]} ],
-              __FILE__, __LINE__, undef, time(),
+              __FILE__, __LINE__, undef
             );
             push @sessions_to_clear, $sid;
           }
@@ -669,7 +687,7 @@ sub _data_sig_reap_pids {
         # Kick off a SIGCHLD cascade.
         $self->_data_ev_enqueue(
           $self, $self, EN_SIGNAL, ET_SIGNAL, [ 'CHLD', $pid, $? ],
-          __FILE__, __LINE__, undef, time(),
+          __FILE__, __LINE__, undef
         );
       }
       elsif (TRACE_SIGNALS) {
@@ -765,9 +783,18 @@ sub _data_sig_kernel_awaits_pids {
 ## the buffer size in the hopes of never getting a short read.  Ever.
 
 use vars qw( $signal_pipe_read_fd );
-my( $signal_pipe_write, $signal_pipe_read, $signal_pipe_pid, 
-    $signal_pipe_len, $signal_pipe_remainder, $signal_pipe_format, 
-    $signal_mask_none, $signal_mask_all, %SIG2NUM, %NUM2SIG );
+my(
+  $signal_pipe_write,
+  $signal_pipe_read,
+  $signal_pipe_pid,
+  $signal_mask_none,
+  $signal_mask_all,
+
+  @pending_signals,
+);
+
+sub SIGINFO_NAME    () { 0 }
+sub SIGINFO_SRC_PID () { 1 }
 
 
 sub _data_sig_pipe_has_signals {
@@ -786,49 +813,25 @@ sub _data_sig_pipe_build {
   return unless USE_SIGNAL_PIPE;
   my $fake = 128;
 
-  $signal_pipe_remainder = '';
-  $signal_pipe_format = "ii";   # pid_t is int on Linux
-  $signal_pipe_len = length pack $signal_pipe_format, 0, 0;
-  
-  unless( %SIG2NUM ) {
-    foreach my $sig ( keys %_safe_signals ) {
-      my $n = eval "POSIX::SIG$sig()";
-      # warn $@;
-      if( $@ ) {    # AKA : RUNNING_IN_HELL
-          # The number used is less important then the fact that it has
-          # a unique number assigned to it
-          $n = $fake++;
-          _trap "<sg> SIG$sig not defined and $n > 255" if $n > 255;
-      }
-      else {
-          # paranoid check
-          _trap "<sg> SIG$sig is out of range ($n)" if $n > 127;
-      }
-      $SIG2NUM{ $sig } = $n;
-      $NUM2SIG{ $n } = $sig;
-    }
-    # warn join "\n", map { "$_: $SIG2NUM{$_}" } sort keys %SIG2NUM;
-    # we need CLD to be named CHLD
-    $SIG2NUM{ CLD } = $SIG2NUM{ CHLD };
-    $NUM2SIG{ $SIG2NUM{ CHLD } } = 'CHLD';
-    $NUM2SIG{ $SIG2NUM{ CLD } } = 'CHLD' if $SIG2NUM{ CLD };
-    # warn join "\n", map { "$_: $_safe_signals{$_}" } sort keys %_safe_signals;
-  }
-
   # Associate the pipe with this PID
   $signal_pipe_pid = $$;
 
   # Mess with the signal mask
   $self->_data_sig_mask_all;
 
-  # Open the signal pipe
+  # Open the signal pipe.
+  # TODO - Normally POE::Pipe::OneWay will do the right thing.  Why
+  # are we overriding its per-platform autodetection?
   if (RUNNING_IN_HELL) {
     ( $signal_pipe_read, $signal_pipe_write ) = POE::Pipe::OneWay->new('inet');
   }
   else {
     ( $signal_pipe_read, $signal_pipe_write ) = POE::Pipe::OneWay->new('pipe');
   }
-  _trap "<sg> Error " . ($!+0) . " trying to create the signal pipe: $!" unless $signal_pipe_write;
+
+  unless ($signal_pipe_write) {
+    _trap "<sg> Error " . ($!+0) . " trying to create the signal pipe: $!";
+  }
 
   # Allows Resource::FileHandles to by-pass the queue
   $signal_pipe_read_fd = fileno $signal_pipe_read;
@@ -894,124 +897,133 @@ sub _data_sig_pipe_finalize {
 ### Send a signal "message" to the main thread
 ### Called from the top signal handlers
 sub _data_sig_pipe_send {
-  my $n = $SIG2NUM{ $_[1] };
-  if( ASSERT_DATA ) {
-    _trap "<sg> Unknown signal $_[1]" unless defined $n;
-  }
+  local $!;
+
+  my $signal_name = $_[1];
+
   if( TRACE_SIGNALS ) {
-    _warn "<sg> Caught SIG$_[1] ($n)";
+    _warn "<sg> Caught SIG$signal_name";
   }
-  
+
   return if $finalizing;
-  
+
   if( not defined $signal_pipe_pid ) {
     _trap "<sg> _data_sig_pipe_send called before signal pipe was initialized.";
   }
+
   # ugh- has_forked() can't be called fast enough.  This warning might
   # show up before it is called.  Should we just detect forking and do it
   # for the user?  Probably not...
+
   if( $$ != $signal_pipe_pid ) {
-    _warn "<sg> Kernel now running in a different process (is=$$ was=$signal_pipe_pid ).  You must call call \$poe_kernel->has_forked in the child process.";
+    _warn(
+      "<sg> Kernel now running in a different process " .
+      "(is=$$ was=$signal_pipe_pid ).  " .
+      "You must call call \$poe_kernel->has_forked() in the child process."
+    );
   }
 
-  my $count = _data_sig_pipe_syswrite( pack( $signal_pipe_format, $$, $n ) );
-  if( ASSERT_DATA ) {
-    if( $count != $signal_pipe_len ) {
-      _trap "<sg> Wrote more than one byte (count=$count)";
+  # We're registering signals in a list.  Pipes have more finite
+  # capacity, so we'll just write a single-byte semaphore-like token.
+  # It's up to the reader to process the list.  Duplicates are
+  # permitted, and their ordering may be significant.  Precedent:
+  # http://search.cpan.org/perldoc?IPC%3A%3AMorseSignals
+
+  push @pending_signals, [
+    $signal_name, # SIGINFO_NAME
+    $$,           # SIGINFO_SRC_PID
+  ];
+
+  if (TRACE_SIGNALS) {
+    _warn "<sg> Attempting signal pipe write";
+  }
+
+  my $count = syswrite( $signal_pipe_write, '!' );
+
+  # TODO - We need to crash gracefully if the write fails, but not if
+  # it's due to the pipe being full.  We might solve this by only
+  # writing on the edge of @pending_signals == 1 after the push().
+  # We assume @pending_signals > 1 means there's a byte in the pipe,
+  # so the reader will wake up to catch 'em all.
+
+  if ( ASSERT_DATA ) {
+    unless (defined $count and $count == 1) {
+      _trap "<sg> Signal pipe write failed: $!";
     }
   }
 }
 
-### write one signal number to the pipe
-sub _data_sig_pipe_syswrite {
-  my( $data ) = @_;
-  my $count = syswrite( $signal_pipe_write, $data );
-  if( defined $count and $count > 0 ) {
-    $! = 0;
-    if( TRACE_SIGNALS ) {
-      _warn "<sg> Wrote $count byte(s) to signal pipe";
-    }
-
-    return $count;
-  }
-
-  # if we got here, something bad happened
-  if( $! == EAGAIN or $! == EWOULDBLOCK ) {
-    _trap "<sg> Excessive signals detected; signal pipe full: $!";
-  }
-  _trap "<sg> Error " . ($!+0) . " writing to signal pipe: $!";
-}
-
 ### Read all signal numbers.
 ### Call the related bottom handler.  That is, inside the kernel loop.
 sub _data_sig_pipe_read {
   my( $self, $fileno, $mode ) = @_;
+
   if( ASSERT_DATA ) {
     _trap "Illegal mode=$mode on fileno=$fileno" unless
                                     $fileno == $signal_pipe_read_fd
                                 and $mode eq MODE_RD;
   }
-  my $data = $self->_data_sig_pipe_sysread();
-  return unless defined $data;
 
-  my $count = length $data;
-  if( TRACE_SIGNALS ) {
-    _warn "<sg> Read $count bytes from signal pipe";
+  # Read all data from the signal pipe.
+  # The data itself doesn't matter.
+  # TODO - If writes can happen on the edge of @pending_signals (from
+  # 0 to 1 element), then we oughtn't need to loop here.
+
+  while (1) {
+    my $octets_count = sysread( $signal_pipe_read, (my $data), 65536 );
+
+    next if $octets_count;
+    last if defined $octets_count;
+
+    last if $! == EAGAIN or $! == EWOULDBLOCK;
+
+    if (ASSERT_DATA) {
+      _trap "<sg> Error " . ($!+0) . " reading from signal pipe: $!";
+    }
+    elsif(TRACE_SIGNALS) {
+      _warn "<sg> Error " . ($!+0) . " reading from signal pipe: $!";
+    }
+
+    last;
   }
-  return unless $count;
 
-  $data = $signal_pipe_remainder . $data;
-  $signal_pipe_remainder = '';
+  # Double buffer signals.
+  # The intent is to avoid a race condition by processing the same
+  # buffer that new signals go into.
 
-  $count = length $data;
-  for(my $q=0; $q< $count; $q+=$signal_pipe_len ) {
-    if( $q + $signal_pipe_len > $count ) {
-        $signal_pipe_remainder = substr( $data, $q );
-        last;
+  return unless @pending_signals;
+  my @signals = @pending_signals;
+  @pending_signals = ();
+
+  if (TRACE_SIGNALS) {
+    _warn "<sg> Read " . scalar(@signals) . " signals from the list";
+  }
+
+  foreach my $signal (@signals) {
+    my $signal_name    = $signal->[SIGINFO_NAME];
+    my $signal_src_pid = $signal->[SIGINFO_SRC_PID];
+
+    # Ignore signals from other processes.
+    # This can happen if we've fork()ed without calling has_forked()
+    # to reset the signals subsystem.
+    #
+    # TODO - We might be able to get rid of has_forked() if PID
+    # mismatches are detected.
+
+    next if $signal_src_pid != $$;
+
+    if( $signal_name eq 'CHLD' ) {
+      _loop_signal_handler_chld_bottom( $signal_name );
     }
-    my($pid, $n) = unpack $signal_pipe_format, substr( $data, $q, $signal_pipe_len );
-    next if $pid != $$; # ignore a 'packet' meant for our parent
-    next if $n == 0;
-    if( ASSERT_DATA ) {
-      _trap "Unknown signal number $n" unless $NUM2SIG{ $n };
-    }
-    my $sig = $NUM2SIG{ $n };
-    if( $sig eq 'CHLD' ) {
-      _loop_signal_handler_chld_bottom( $sig );
-    }
-    elsif( $sig eq 'PIPE' ) {
-      _loop_signal_handler_pipe_bottom( $sig );
+    elsif( $signal_name eq 'PIPE' ) {
+      _loop_signal_handler_pipe_bottom( $signal_name );
     }
     else {
-      _loop_signal_handler_generic_bottom( $sig );
+      _loop_signal_handler_generic_bottom( $signal_name );
     }
   }
 }
 
-### Read all signal numbers from the pipe
-sub _data_sig_pipe_sysread {
-  my $data = '';
-  # To avoid flooding the queue, we don't read the entire pipe at once
-  # PG- Mind you, I doubt signal flooding is ever going to be much of
-  # a problem, is it?
-  my $result = sysread( $signal_pipe_read, $data, 256*$signal_pipe_len ); # XXX
-  if( defined $result ) {
-    $! = 0;
-    return $data;
-  }
-  # Nonfatal sysread() error.  Return an empty list.
-  return '' if $! == EAGAIN or $! == EWOULDBLOCK;
-
-  if( ASSERT_DATA ) {
-    _trap "<sg> Error " . ($!+0) . " reading from signal pipe: $!";
-  }
-  elsif( TRACE_SIGNALS ) {
-    _warn "<sg> Error " . ($!+0) . " reading from signal pipe: $!";
-  }
-  return;
-}
-
-
 1;
 
 __END__
@@ -1038,7 +1050,7 @@ handling.
 See L<POE::Kernel/Signal Watcher Methods> for POE's public signals
 API.
 
-See L<POE::Kernel/Resources> for for public information about POE
+See L<POE::Kernel/Resources> for public information about POE
 resources.
 
 See L<POE::Resource> for general discussion about resources and the
